/*
 * EnemyWave.h
 *
 *  Created on: Jan 28, 2012
 *      Author: Santi Ontanon
 *  Description:
 *      - In knoghtmare, enemies appear in waves. Each wave consists of a series of groups of enemies
 *      - Each group of enemies contains enemies of the same type
 *      - The next wave of enemies does not appear until the last wave has been cleared
 */

#ifdef KITSCHY_DEBUG_MEMORY
#include "debug_memorymanager.h"
#endif

#ifdef _WIN32
#include "windows.h"
#endif

#include "math.h"
#include "stdio.h"

#include "SDL.h"
#include "SDL_image.h"

#include "GLTile.h"
#include "debug.h"
#include "XMLparser.h"

#include "Knightmare.h"
#include "KMap.h"

#include "KObject.h"
#include "Blob.h"
#include "Bat.h"

#include "KObjectCreator.h"

#include "EnemyWave.h"


void EnemyWave::addGroup(EnemyGroup *g) {
        m_groups.Add(g);
}


bool EnemyWave::cycle(KMap *map, Knightmare *game, GLTManager *GLTM, SFXManager *SFXM) {
    EnemyGroup *group;
    m_groups.Rewind();
    while(m_groups.Iterate(group)) {
        if (group->m_type->cmp(blob_symbol) && group->m_number>0) {
            if (group->m_number>0) {
                if ((map->get_cycle()%128)==0) {
                    KObject *o = new Blob(map->get_tile_width()*5 + (rand()%22)*map->get_tile_width(),
                                          map->get_current_y()-2*map->get_tile_height()*3,game->get_sfx_volume(),GLTM);
                    map->addObject(2,o);
                    m_enemies.Add(o);
                    group->m_number--;
                }
            }      
            return false;
        } else if (group->m_type->cmp(bat_symbol)) {
            if (group->m_number>0) {
                if ((map->get_cycle()%16)==0) {
                    KObject *o = new Bat(map->get_tile_width()*16,
                                         map->get_current_y()-3*map->get_tile_height(),game->get_sfx_volume(),GLTM);
                    map->addObject(2,o);
                    m_enemies.Add(o);
                    group->m_number--;
                }
                return false;
            }
        }
    }
    
    return waveClearedP(map);
}

    
bool EnemyWave::waveClearedP(KMap *map) {
    List<KObject> toDelete,l;
    l.Instance(m_enemies);
    l.Rewind();
    class KObject *o;
    while(l.Iterate(o)) {
        if (!map->objectExists(o)) {
            toDelete.Add(o);
        }
    }
    while(!toDelete.EmptyP()) {
        o = toDelete.ExtractIni();
        m_enemies.DeleteElement(o);
    }
    return m_enemies.EmptyP();
}

