/*
 * EnemyWave.h
 *
 *  Created on: Jan 28, 2012
 *      Author: Santi Ontanon
 *  Description:
 *      - In knoghtmare, enemies appear in waves. Each wave consists of a series of groups of enemies
 *      - Each group of enemies contains enemies of the same type
 *      - The next wave of enemies does not appear until the last wave has been cleared
 */

#ifndef ENEMYWAVE_H_
#define ENEMYWAVE_H_

#include "BList.h"
#include "Symbol.h"
#include "KObject.h"
#include "K3DObject.h"


class EnemyGroup {
public:
	EnemyGroup(Symbol *type, Symbol *behavior, int number) {
		m_type = type;
		m_behavior = behavior;
		m_number = number;
	}
    
	~EnemyGroup() {
		delete m_type;
        if (m_behavior!=0) delete m_behavior;
	}
    
	Symbol *m_type, *m_behavior;
	int m_number;
};



class EnemyWave {
public:
	EnemyWave() {
	}

	~EnemyWave() {
        m_enemies.ExtractAll();
	}
    
    void addGroup(EnemyGroup *g);
    
    
    bool cycle(class KMap *map, class Knightmare *game, class GLTManager *GLTM, class SFXManager *SFXM);
    bool waveClearedP(KMap *map);
    
    List<EnemyGroup> m_groups;
    List<KObject> m_enemies;        // All the enemies that have already been generated are stored in this list 
                                    // to keep track of them.
};

#endif /* ENEMYWAVE_H_ */
